import os
from pydub import AudioSegment
from pyannote.audio import Pipeline
import torch

# Define your environment variable here
os.environ["YOUR_TOKEN_NAME"] = "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"

# Static variables
input_folder = r'F:\coqui_test\training_dataset\wavs'
output_folder = r'F:\coqui_test\training_dataset\other_speaker'
reference_file = 'sound-100-10116.wav'  # Only the file name

# Create the output folder if it does not exist
if not os.path.exists(output_folder):
    os.makedirs(output_folder)

# Step 3: Combine WAV Files and Track Durations
combined_audio = AudioSegment.empty()
file_durations = {}
current_duration = 0.0
total_files = 0

# Iterate over wav files in the input folder and combine them
for file_name in os.listdir(input_folder):
    if file_name.endswith('.wav'):
        total_files += 1
        file_path = os.path.join(input_folder, file_name)
        audio = AudioSegment.from_wav(file_path)
        combined_audio += audio

        file_durations[file_name] = (current_duration, current_duration + len(audio) / 1000.0)
        current_duration += len(audio) / 1000.0

# Export the combined audio to a temporary file
combined_audio_path = 'combined_audio.wav'
combined_audio.export(combined_audio_path, format='wav')

# Step 4: Speaker Classification
pipeline = Pipeline.from_pretrained("pyannote/speaker-diarization-3.1")
pipeline.to(torch.device("cuda"))

diarization = pipeline(combined_audio_path)

# Process the diarization results
speaker_times = {}
for turn, _, speaker in diarization.itertracks(yield_label=True):
    if speaker not in speaker_times:
        speaker_times[speaker] = []
    speaker_times[speaker].append((turn.start, turn.end))

# Display the number of speakers found
print(f"Number of speakers found: {len(speaker_times)}")

# Step 5: Identify Speaker ID for Reference File
reference_start, reference_end = file_durations[reference_file]
reference_speaker = None
for speaker, times in speaker_times.items():
    for start, end in times:
        if start <= reference_start and end >= reference_end:
            reference_speaker = speaker
            break
    if reference_speaker is not None:
        break

# Display which speaker corresponds to the reference file
print(f"Reference file corresponds to Speaker ID: {reference_speaker}")

# Step 6: Move Files Based on Speaker ID
moved_files_count = 0
for file_name, (start, end) in file_durations.items():
    if file_name != reference_file:  # To avoid moving the reference file itself
        for speaker, times in speaker_times.items():
            if speaker != reference_speaker:
                for s_start, s_end in times:
                    if s_start <= start and s_end >= end:
                        os.rename(os.path.join(input_folder, file_name),
                                  os.path.join(output_folder, file_name))
                        moved_files_count += 1
                        break

    # Display status
    print(f"Processed {file_name}. {moved_files_count} files moved. {total_files - moved_files_count} files remaining.")

# Final status
print(f"Completed processing. Total files moved: {moved_files_count}")
