import os
import requests

def create_directory(path):
    """Create a directory if it does not exist."""
    if not os.path.exists(path):
        os.makedirs(path)
        print(f"Directory '{path}' created.")
    else:
        print(f"Directory '{path}' already exists.")

def download_file(url, folder):
    """Download a file from a URL to a specified folder and show download percentage."""
    local_filename = url.split('/')[-1]
    path = os.path.join(folder, local_filename)

    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        total_length = int(r.headers.get('content-length'))
        downloaded = 0

        with open(path, 'wb') as f:
            for chunk in r.iter_content(chunk_size=8192):
                if chunk:
                    f.write(chunk)
                    downloaded += len(chunk)
                    done = int(50 * downloaded / total_length)
                    print(f"Downloading {local_filename}: [{'=' * done}{' ' * (50-done)}] {downloaded * 100 / total_length:.2f}%", end='\r')
    print(f"\nDownloaded {local_filename} to {folder}")

def main():
    base_folder = 'TTS_models'
    sub_folder = 'XTTS_v2.0_original_model_files'
    full_path = os.path.join(base_folder, sub_folder)

    create_directory(base_folder)
    create_directory(full_path)

    files_to_download = [
        "https://huggingface.co/coqui/XTTS-v2/resolve/main/dvae.pth",
        "https://huggingface.co/coqui/XTTS-v2/resolve/main/config.json",
        "https://huggingface.co/coqui/XTTS-v2/resolve/main/model.pth",
        "https://huggingface.co/coqui/XTTS-v2/resolve/main/mel_stats.pth",
        "https://huggingface.co/coqui/XTTS-v2/resolve/main/vocab.json"
    ]

    for file_url in files_to_download:
        try:
            download_file(file_url, full_path)
        except requests.exceptions.RequestException as e:
            print(f"Error downloading {file_url}: {e}")

if __name__ == "__main__":
    main()
