import sys
import os
from pydub import AudioSegment
from pyannote.audio import Pipeline
import torch

# Define your environment variable here
os.environ["YOUR_TOKEN_NAME"] = "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"

# Static variables
input_folder = r'F:\coqui_test\training_dataset\wavs'
output_folder = r'F:\coqui_test\training_dataset\other_speaker'
reference_file = 'sound-100-10116.wav'  # Only the file name

# Create the output folder if it does not exist
if not os.path.exists(output_folder):
    os.makedirs(output_folder)

# Step 3: Combine WAV Files and Track Durations
combined_audio = AudioSegment.empty()
file_durations = {}
current_duration = 0.0
total_files = 0
print("generating combined audio may take a while...")
# Iterate over wav files in the input folder and combine them
combined_counter = 0
for file_name in os.listdir(input_folder):
    if file_name.endswith('.wav'):
        total_files += 1
        file_path = os.path.join(input_folder, file_name)
        audio = AudioSegment.from_wav(file_path)
        combined_audio += audio
        file_durations[file_name] = (current_duration, current_duration + len(audio) / 1000.0)
        current_duration += len(audio) / 1000.0
        combined_counter += 1
        # Update the counter on the same line
        print(f"Combined audio counter: {combined_counter}", end='\r')
        sys.stdout.flush()

# Export the combined audio to a temporary file
combined_audio_path = 'combined_audio.wav'
combined_audio.export(combined_audio_path, format='wav')

# Step 4: Speaker Classification
pipeline = Pipeline.from_pretrained("pyannote/speaker-diarization-3.1")
pipeline.to(torch.device("cuda"))

print("diarization of combined audio may take a while...")
diarization = pipeline(combined_audio_path)

# Process the diarization results
speaker_times = {}
for turn, _, speaker in diarization.itertracks(yield_label=True):
    if speaker not in speaker_times:
        speaker_times[speaker] = []
    speaker_times[speaker].append((turn.start, turn.end))

# Display the number of speakers found
print(f"Number of speakers found: {len(speaker_times)}")

# Step 5: Identify Speaker ID for Reference File
reference_start, reference_end = file_durations[reference_file]
reference_speaker = None
for speaker, times in speaker_times.items():
    for start, end in times:
        if start <= reference_start and end >= reference_end:
            reference_speaker = speaker
            break
    if reference_speaker is not None:
        break
if reference_speaker is None:
    for speaker, times in speaker_times.items():
        for start, end in times:
            if start >= reference_start and end <= reference_end:
                reference_speaker = speaker
                break
        if reference_speaker is not None:
            break

# Display which speaker corresponds to the reference file
print(f"Reference file corresponds to Speaker ID: {reference_speaker}")

# Step 6: Move Files Based on Speaker Identification with Improved Variable Names
total_moved_files_count = 0
for individual_file_name, (segment_start_time, segment_end_time) in file_durations.items():
    if individual_file_name != reference_file:  # To avoid moving the reference file itself
        for detected_speaker, speaker_time_segments in speaker_times.items():
            if detected_speaker != reference_speaker:
                for speaker_segment_start, speaker_segment_end in speaker_time_segments:
                    # Check for any overlap between the file's time segment and the speaker's time segment
                    if (speaker_segment_start < segment_end_time and speaker_segment_end > segment_start_time):
                        source_file_path = os.path.join(input_folder, individual_file_name)
                        destination_file_path = os.path.join(output_folder, individual_file_name)
                        # Check if the source file exists before renaming
                        if os.path.exists(source_file_path):
                            os.rename(source_file_path, destination_file_path)
                            total_moved_files_count += 1
                            break


    # Display status
    print(f"Processed {individual_file_name}. {total_moved_files_count} files moved. {total_files - total_moved_files_count} files remaining.")

# Final status
print(f"Completed processing. Total files moved: {total_moved_files_count}")

