import os
import whisper

# Static variables - Set these before running the script
INPUT_FOLDER = r"F:\coqui_test\training_dataset\wavs"
OUTPUT_FOLDER = r"F:\coqui_test\training_dataset"
LANGUAGE = "en"

# Ensure the output folder exists
os.makedirs(OUTPUT_FOLDER, exist_ok=True)

def transcribe_audio(model, input_file, language):
    print(f"Transcribing audio in {input_file}...")
    result = model.transcribe(input_file, language=language)
    return result["text"].strip()

def process_files():
    print("Loading Whisper model...")
    model = whisper.load_model("large-v3")
    metadata = []
    file_counter = 1

    # Process all .wav files in the input folder
    all_files = [f for f in os.listdir(INPUT_FOLDER) if f.endswith(".wav")]
    total_files = len(all_files)
    print(f"Total files to process: {total_files}")

    for filename in all_files:
        print(f"Processing file {file_counter} of {total_files}: {filename}")
        file_path = os.path.join(INPUT_FOLDER, filename)
        transcription = transcribe_audio(model, file_path, LANGUAGE)
        # Duplicate the transcription as per the requirement
        doubled_transcription = f"{transcription}|{transcription}"
        metadata.append((filename[:-4], doubled_transcription))  # Removing the .wav extension
        print(f"File {file_counter} transcribed.")
        file_counter += 1

    print("Writing metadata to CSV...")
    with open(os.path.join(OUTPUT_FOLDER, 'metadata.csv'), 'w', encoding='utf-8', newline='') as csvfile:
        for filename, transcription in metadata:
            line = f"{filename}|{transcription}\n"
            csvfile.write(line)
    print("Data processing complete.")

# Run the script
process_files()
